function param_estimator = calcul_param_fitzHugh(bs,theta_ini,func_test,L,Weight)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Parameter estimation 
%
%USAGE:  param_estimator = calcul_param_fitzHugh(bs,theta_ini,func_test,L,Weight)
%
%INPUT :
%bs              :(Non-parametric curve estimator  object) , embedding the curve estimator
%theta_ini     	 : (vector) first guess for parameter values
%func_test         :(function handler) used test function (try @splines_annulateur_borne)
%L               : (integer) number of test functions 
%(Optionnal)     :
%Weight          : weight matrix (default identity matrix)
%OUTPUT:
%param_estimator     :(matrix) result of the parameter estimation             
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            obs =  bs.getObservation();         
            T_gen = obs.getTimes();
            TS = T_gen{1};
            
            T_0 = TS(1);
            T_1 = TS(length(TS));
            
            if nargin < 5
                Weight = eye(L);
            end


    function res_aux = coeff_ftd_seconde(t)

         fonc_der_sec_test = func_test(t,T_0,T_1,L,2,1)';        
         X_pred = bs.calcul_prediction(t,0);

         res_aux = zeros(L,1);

             for line =  1 : L                 
                 res_aux(line) = X_pred(1)*fonc_der_sec_test(line);
             end
    end

    function res_aux = coeff_ftd_premiere(t)

         fonc_der_test = func_test(t,T_0,T_1,L,1,1)';        
         X_pred = bs.calcul_prediction(t,0);

         res_aux = zeros(L,1);

             for line =  1 : L            
            res_aux(line) = X_pred(1)*fonc_der_test(line);
             end
    end

     function res_aux = coeff_ftd_premiere_cube(t)

         fonc_der_test = func_test(t,T_0,T_1,L,1,1)';        
         X_pred = bs.calcul_prediction(t,0);

         res_aux = zeros(L,1);

             for line =  1 : L            
             res_aux(line) = (X_pred(1)-(X_pred(1)^3)/3)*fonc_der_test(line);
             end
    end

    function res_aux = coeff_ftd_zeros(t)

         fonc_dir_test = func_test(t,T_0,T_1,L,0,1)';        
         X_pred = bs.calcul_prediction(t,0);

         res_aux = zeros(L,1);

             for line =  1 : L            
                 res_aux(line,1) = X_pred(1)*fonc_dir_test(line);
             end
    end

  function res_aux = coeff_ftd_zeros_cube(t)

         fonc_dir_test = func_test(t,T_0,T_1,L,0,1)';        
         X_pred = bs.calcul_prediction(t,0);

         res_aux = zeros(L,1);

             for line =  1 : L            
                 res_aux(line,1) = ((X_pred(1).^3)/3)*fonc_dir_test(line);
             end
  end

    function res_aux = coeff_ftd(t)

        res_aux = func_test(t,T_0,T_1,L,0,1)';        

    end


der_sec = quadv(@coeff_ftd_seconde,T_0,T_1);
der_prem = quadv(@coeff_ftd_premiere,T_0,T_1);
der_zeros = quadv(@coeff_ftd_zeros,T_0,T_1);
der_prem_cube = quadv(@coeff_ftd_premiere_cube,T_0,T_1);
der_zeros_cube = quadv(@coeff_ftd_zeros_cube,T_0,T_1);
fonction_test = quadv(@coeff_ftd,T_0,T_1);

    function [res_scalaire Gradient] = func_eco_gen(theta_gen,ftd_sec,ftd_der,ftd_zeros,ftd_prem_cube,ftd_zeros_cube,ftd)
        
        a = theta_gen(1);
        b = theta_gen(2);
        c = theta_gen(3);
        
        linear_part_der1 = -(b/c)*ftd_der+ c*ftd_prem_cube;       
        linear_part_zeros1 = (1-b)*ftd_zeros+b*ftd_zeros_cube -a*ftd;
        
        
        
        res = ftd_sec+linear_part_der1+linear_part_zeros1;
        
        res_scalaire = res'*Weight*res;
        
        if nargout > 1
            
            Gradient = zeros(3,1);
            Jacobien_integ = zeros(L,3);
            Jacobien_integ(:,1) = -ftd';
            Jacobien_integ(:,2) = -(1/c)*ftd_der' +ftd_zeros_cube'-ftd_zeros';
            Jacobien_integ(:,3) = ftd_prem_cube' + (b/(c^2))*ftd_der';
            
            
         for nb_cond_i = 1: L
                     for nb_cond_j = 1 : L

                   Jacob_i = Jacobien_integ(nb_cond_i,:)';     
                   Jacob_j = Jacobien_integ(nb_cond_j,:)';  
                   Gradient =  Gradient + Weight(nb_cond_i,nb_cond_j)*(Jacob_j*vecteur(nb_cond_i)+Jacob_i*vecteur(nb_cond_j));
                   %Gradient =  Gradient + (Jacob_j*res(nb_cond_i)+Jacob_i*res(nb_cond_j));
                     end
         end
        end
    end

    function [res_opt grad_opt] = func_eco(param)
       %param
        if nargout > 1
             [res_opt grad_opt] = func_eco_gen(param,der_sec,der_prem,der_zeros,der_prem_cube,der_zeros_cube,fonction_test);
        else
              res_opt= func_eco_gen(param,der_sec,der_prem,der_zeros,der_prem_cube,der_zeros_cube,fonction_test);
        end
    end
    options = optimset('TolFun',1e-10,'TolX',1e-10,'GradObj','off','MaxFunEvals',inf,'MaxIter',10000,'Hessian','off');
    %param_estimator = fmincon(@func_eco,theta_ini,Ain,bin,Aeq,beq,lb,ub,[],options);
    %param_estimator = func_eco(theta_ini)
    param_estimator = fminunc(@func_eco,theta_ini,options);

end