function [T X] = fitz_nagumo(deb,fin,h,y0,a,b,c,std,control)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % generate noisy data from the FitzHugh-Nagumo model (presented in thesis manuscript introduction)
%
%USAGE:  [T X] = fitz_nagumo(deb,fin,h,y0,a,b,c,std,control)
%
%INPUT :
%deb                        :(real) lower bound for the observation interval
%fin                        :(real) upper bound for the observation interval
%h                          :(real) step for the data generation
%y0                         :(vector) IC value
%a                          :(real) correspond to parameter a in the model  
%b                          :(real) correspond to parameter b in the model 
%c                          :(real) correspond to parameter c in the model  
%std                        :(real) standart deviation of the added gaussian measurement error
%OPTIONAL:
%control                    :(function handler default value: a function returning 0) added pertubation to the
%                              initial model in order to generate artificial misspecification (in default setting no perturbation added)                       
%OUTPUT
%T                          :(vector) generated observation times
%X                          :(matrix) generated noised observations
%               
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

pres_control = 0;
if nargin > 8
    pres_control= 1;
end

temp = [deb :h :fin];
T = temp;

function dy = equ(t,y)
dy = zeros(2,1);
dy(1) = c*(y(1) - (y(1)^3)/3 + y(2));
dy(2) = -(1/c)*(y(1) - a + b*y(2));

if pres_control == 1
    dy = dy + control(t);
end

end

if nargout > 1 
[T,Y] = ode45(@equ,temp,y0);

res = Y + std*randn(length(temp),2);

X =  res;

else
    sol = ode45(@equ,temp,y0);
    T = sol;
end

%plot(T,Y(:,1),'b',T,res(:,1),'bo');
%plot(T,Y(:,1),'b',T,res(:,1),'bo',T,Y(:,2),'r',T,res(:,2),'rx');
%plot(T,Y(:,1),'b',T,Y(:,2),'r');
%plot(T,res(:,1),'bx',T,res(:,2),'rx');

end
