function [res, T_plot] = log_influenza_nus_sim(a,pas,b,theta,C_I,bs_dend,delay,std,time_shift,g_l_function,nu_s_function)
%DESCRIPTION: %Simulate log data for cell growth and migration model
%
%USAGE:   [res, T_plot] = log_influenza_nus_sim(a,pas,b,theta,C_I,bs_dend,delay,std,time_shift,g_l_function,nu_s_function)
%INPUT :
%a                :(real) beginning of the observation interval
%pas              :(integer) number of observation required
%b                :(real) end of the observation interval
%theta            :(vector) vector of parameters
%C_I              :(vector) initial conditions for the system
%bs_dend          :(bSpline object) dendritic cell function
%delay            :(real) delay tau in dendritic cell function
%std              :(positive real) variance level of measurement noise
%time_shift       :(integer) shift the observation interval
%g_l_function     :(function handler): functional parameter delta_l
%nu_s_function    :(function handler): exogenous perturbation, influx of CD8+T cells coming from  additional compartments than the spleen.
%OUTPUT:
%res:                 :(matrix or matlab solution object) simulated model at timepoints precised in point_interet OR matlab solution object if nargout == 1
%T_plot               :(vector) vector of observation time          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

   if nargin > 9 
        p_m = theta(1);
        p_s = theta(2);
        g_l = g_l_function;

        g_ms = theta(3);
        g_sl = theta(4);
   else
    	p_m = theta(1);
        p_s = theta(2);
        g_l = theta(3);

        g_ms = theta(4);
        g_sl = theta(5);
   end
    

    
    function res_temp = dend_cell(t)
        res_temp = bs_dend.calcul_prediction(t);
        res_temp = arrayfun(@(x)10^x,res_temp);
    end
    
    function dy = equ(t,y)
      dy = zeros(2+1,1);
      
      nu_s_t = nu_s_function(t);
      
      if isa(g_l,'function_handle')
          gl_t = g_l(t);
      else
          gl_t = g_l;
      end
      
        dy(1) = (p_m*dend_cell(t+time_shift-delay)-g_ms);
        dy(2) = (p_s*dend_cell(t+time_shift-delay)-g_sl)+g_ms*exp(y(1)-y(2))+ nu_s_t*exp(-y(2));
        dy(3) = g_sl*exp(y(2)-y(3)) -gl_t;
        
    end

    vecteur = a:(b-a)/pas:b;
    [T X] = ode45(@equ,vecteur,C_I);
    
   
    
    T_plot = T;
    
    %plot(T,X(:,1),'r',T,X(:,2),'g',T,X(:,3),'b')
    res = X + std*randn(length(T_plot),3);
            
     if nargout < 2
         res =ode45(@equ,vecteur,C_I);
     end
 end