function sol = pontryagin_compute_controlled_solution(bs_u,vect_field,est_param,x_0,mat_b,param_fixe)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%DESCRIPTION: % Compute the solution of the forced ODE dx/dt = f(t,x,theta)+Bu 
%
%USAGE:     sol = pontryagin_compute_controlled_solution(bs_u,vect_field,est_param,x_0,mat_b,param_fixe)
%
%INPUT :
%bs_u             :(bSplines  object) , embed the nonparametric estimators
%vect_field       :(function handler) embed the function corresponding to the vector field with signature: 
%                  res_t =  vect_field(t,vx,param,param_fixe)
%                           t : time
%                           vx : state value at time t
%                           param : varying parameter value for the plotted graph
%                           param_fixe : other parameter value
%                           res_t : (vector of size dim x 1) vector field  value at time t
%est_param                : (vector of size nb param) parameter value
%x_0                      : (vector) initial condition 
%mat_b                    : (matrix) matrix B in the forced ODE
%OPTIONAL
%param_fixe               :(vector) fixed parameter for the model (default empty ([]) )
%OUTPUT:
%sol                     : (matlab ODE object) solution of the forced ODE             
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

    Obs = bs_u.getObservation();
    T_c = Obs.getTimes();
    Times = T_c{1};
    dim_syst = length(x_0);

    pas_solve =400;
    Int_ode_solve = Times(1):((Times(end)-Times(1))/pas_solve):Times(end); 
    
    if nargin < 6
        param_fixe = [];
    end
    
    if nargin < 5
        mat_b = eye(dim_syst);
    end
    
    dim_control =  length(mat_b(1,:));

    function res_t = champ_vect(t,vx)
        res_u_t = bs_u.calcul_prediction(t,0);
        res_t = vect_field(t,vx,est_param,param_fixe)+ mat_b*reshape(res_u_t,dim_control,1);
    end
       
    sol = ode45(@champ_vect,Int_ode_solve,reshape(x_0,dim_syst,1));
   
end